@@ -2,15 +2,18 @@ from PyQt5.QtWidgets import QFileDialog
 
 from electrum.i18n import _
 from electrum.plugin import run_hook
+from electrum.simple_config import SimpleConfig
+from electrum.util import UserFacingException
 
-from .util import ButtonsTextEdit, MessageBoxMixin, ColorScheme, get_parent_main_window
+from .util import ButtonsTextEdit, MessageBoxMixin, ColorScheme, getOpenFileName
 
 
 class ShowQRTextEdit(ButtonsTextEdit):
 
-    def __init__(self, text=None):
+    def __init__(self, text=None, *, config: SimpleConfig):
         ButtonsTextEdit.__init__(self, text)
-        self.setReadOnly(1)
+        self.config = config
+        self.setReadOnly(True)
         icon = "qrcode_white.png" if ColorScheme.dark_scheme else "qrcode.png"
         self.addButton(icon, self.qr_show, _("Show as QR code"))
 
@@ -22,7 +25,11 @@ class ShowQRTextEdit(ButtonsTextEdit):
             s = str(self.toPlainText())
         except:
             s = self.toPlainText()
-        QRDialog(s, parent=self).exec_()
+        QRDialog(
+            data=s,
+            parent=self,
+            config=self.config,
+        ).exec_()
 
     def contextMenuEvent(self, e):
         m = self.createStandardContextMenu()
@@ -32,17 +39,22 @@ class ShowQRTextEdit(ButtonsTextEdit):
 
 class ScanQRTextEdit(ButtonsTextEdit, MessageBoxMixin):
 
-    def __init__(self, text="", allow_multi=False):
+    def __init__(self, text="", allow_multi=False, *, config: SimpleConfig):
         ButtonsTextEdit.__init__(self, text)
         self.allow_multi = allow_multi
-        self.setReadOnly(0)
+        self.config = config
+        self.setReadOnly(False)
         self.addButton("file.png", self.file_input, _("Read file"))
         icon = "camera_white.png" if ColorScheme.dark_scheme else "camera_dark.png"
         self.addButton(icon, self.qr_input, _("Read QR code"))
         run_hook('scan_text_edit', self)
 
     def file_input(self):
-        fileName, __ = QFileDialog.getOpenFileName(self, 'select file')
+        fileName = getOpenFileName(
+            parent=self,
+            title='select file',
+            config=self.config,
+        )
         if not fileName:
             return
         try:
@@ -60,14 +72,14 @@ class ScanQRTextEdit(ButtonsTextEdit, MessageBoxMixin):
 
     def qr_input(self):
         from electrum import qrscanner
-        window_or_wizard = get_parent_main_window(self, allow_wizard=True)
-        assert window_or_wizard
-        config = window_or_wizard.config
+        data = ''
         try:
-            data = qrscanner.scan_barcode(config.get_video_device())
+            data = qrscanner.scan_barcode(self.config.get_video_device())
+        except UserFacingException as e:
+            self.show_error(e)
         except BaseException as e:
+            self.logger.exception('camera error')
             self.show_error(repr(e))
-            data = ''
         if not data:
             data = ''
         if self.allow_multi:
