@@ -37,7 +37,8 @@ from .invoices import PR_TYPE_ONCHAIN, Invoice
 from .keystore import bip44_derivation
 from .transaction import Transaction, TxOutpoint, tx_from_any, PartialTransaction, PartialTxOutput
 from .logging import Logger
-from .lnutil import LOCAL, REMOTE, FeeUpdate, UpdateAddHtlc, LocalConfig, RemoteConfig, Keypair, OnlyPubkeyKeypair, RevocationStore, ChannelBackupStorage
+from .lnutil import LOCAL, REMOTE, FeeUpdate, UpdateAddHtlc, LocalConfig, RemoteConfig, Keypair, OnlyPubkeyKeypair, RevocationStore
+from .lnutil import ImportedChannelBackupStorage, OnchainChannelBackupStorage
 from .lnutil import ChannelConstraints, Outpoint, ShachainElement
 from .json_db import StoredDict, JsonDB, locked, modifier
 from .plugin import run_hook, plugin_loaders
@@ -52,7 +53,7 @@ if TYPE_CHECKING:
 
 OLD_SEED_VERSION = 4        # electrum versions < 2.0
 NEW_SEED_VERSION = 11       # electrum versions >= 2.0
-FINAL_SEED_VERSION = 32     # electrum >= 2.7 will set this to prevent
+FINAL_SEED_VERSION = 40     # electrum >= 2.7 will set this to prevent
                             # old versions from overwriting new format
 
 
@@ -180,6 +181,14 @@ class WalletDB(JsonDB):
         self._convert_version_30()
         self._convert_version_31()
         self._convert_version_32()
+        self._convert_version_33()
+        self._convert_version_34()
+        self._convert_version_35()
+        self._convert_version_36()
+        self._convert_version_37()
+        self._convert_version_38()
+        self._convert_version_39()
+        self._convert_version_40()
         self.put('seed_version', FINAL_SEED_VERSION)  # just to be sure
 
         self._after_upgrade_tasks()
@@ -520,10 +529,10 @@ class WalletDB(JsonDB):
             r['buckets'] = d
             c['revocation_store'] = r
         # convert channels to dict
-        self.data['channels'] = { x['channel_id']: x for x in channels }
+        self.data['channels'] = {x['channel_id']: x for x in channels}
         # convert txi & txo
         txi = self.get('txi', {})
-        for tx_hash, d in txi.items():
+        for tx_hash, d in list(txi.items()):
             d2 = {}
             for addr, l in d.items():
                 d2[addr] = {}
@@ -532,7 +541,7 @@ class WalletDB(JsonDB):
             txi[tx_hash] = d2
         self.data['txi'] = txi
         txo = self.get('txo', {})
-        for tx_hash, d in txo.items():
+        for tx_hash, d in list(txo.items()):
             d2 = {}
             for addr, l in d.items():
                 d2[addr] = {}
@@ -687,14 +696,121 @@ class WalletDB(JsonDB):
     def _convert_version_32(self):
         if not self._is_upgrade_method_needed(31, 31):
             return
-
-        from .invoices import PR_TYPE_ONCHAIN
+        PR_TYPE_ONCHAIN = 0
         invoices_old = self.data.get('invoices', {})
         invoices_new = {k: item for k, item in invoices_old.items()
                         if not (item['type'] == PR_TYPE_ONCHAIN and item['outputs'] is None)}
         self.data['invoices'] = invoices_new
         self.data['seed_version'] = 32
 
+    def _convert_version_33(self):
+        if not self._is_upgrade_method_needed(32, 32):
+            return
+        PR_TYPE_ONCHAIN = 0
+        requests = self.data.get('payment_requests', {})
+        invoices = self.data.get('invoices', {})
+        for d in [invoices, requests]:
+            for key, item in list(d.items()):
+                if item['type'] == PR_TYPE_ONCHAIN:
+                    item['height'] = item.get('height') or 0
+        self.data['seed_version'] = 33
+
+    def _convert_version_34(self):
+        if not self._is_upgrade_method_needed(33, 33):
+            return
+        channels = self.data.get('channels', {})
+        for key, item in channels.items():
+            item['local_config']['upfront_shutdown_script'] = \
+                item['local_config'].get('upfront_shutdown_script') or ""
+            item['remote_config']['upfront_shutdown_script'] = \
+                item['remote_config'].get('upfront_shutdown_script') or ""
+        self.data['seed_version'] = 34
+
+    def _convert_version_35(self):
+        # same as 32, but for payment_requests
+        if not self._is_upgrade_method_needed(34, 34):
+            return
+        PR_TYPE_ONCHAIN = 0
+        requests_old = self.data.get('payment_requests', {})
+        requests_new = {k: item for k, item in requests_old.items()
+                        if not (item['type'] == PR_TYPE_ONCHAIN and item['outputs'] is None)}
+        self.data['payment_requests'] = requests_new
+        self.data['seed_version'] = 35
+
+    def _convert_version_36(self):
+        if not self._is_upgrade_method_needed(35, 35):
+            return
+        old_frozen_coins = self.data.get('frozen_coins', [])
+        new_frozen_coins = {coin: True for coin in old_frozen_coins}
+        self.data['frozen_coins'] = new_frozen_coins
+        self.data['seed_version'] = 36
+
+    def _convert_version_37(self):
+        if not self._is_upgrade_method_needed(36, 36):
+            return
+        payments = self.data.get('lightning_payments', {})
+        for k, v in list(payments.items()):
+            amount_sat, direction, status = v
+            amount_msat = amount_sat * 1000 if amount_sat is not None else None
+            payments[k] = amount_msat, direction, status
+        self.data['lightning_payments'] = payments
+        self.data['seed_version'] = 37
+
+    def _convert_version_38(self):
+        if not self._is_upgrade_method_needed(37, 37):
+            return
+        PR_TYPE_ONCHAIN = 0
+        PR_TYPE_LN = 2
+        from .bitcoin import TOTAL_COIN_SUPPLY_LIMIT_IN_BTC, COIN
+        max_sats = TOTAL_COIN_SUPPLY_LIMIT_IN_BTC * COIN
+        requests = self.data.get('payment_requests', {})
+        invoices = self.data.get('invoices', {})
+        for d in [invoices, requests]:
+            for key, item in list(d.items()):
+                if item['type'] == PR_TYPE_ONCHAIN:
+                    amount_sat = item['amount_sat']
+                    if amount_sat == '!':
+                        continue
+                    if not (isinstance(amount_sat, int) and 0 <= amount_sat <= max_sats):
+                        del d[key]
+                elif item['type'] == PR_TYPE_LN:
+                    amount_msat = item['amount_msat']
+                    if not amount_msat:
+                        continue
+                    if not (isinstance(amount_msat, int) and 0 <= amount_msat <= max_sats * 1000):
+                        del d[key]
+        self.data['seed_version'] = 38
+
+    def _convert_version_39(self):
+        # this upgrade prevents initialization of lightning_privkey2 after lightning_xprv has been set
+        if not self._is_upgrade_method_needed(38, 38):
+            return
+        self.data['imported_channel_backups'] = self.data.pop('channel_backups', {})
+        self.data['seed_version'] = 39
+
+    def _convert_version_40(self):
+        # put 'seed_type' into keystores
+        if not self._is_upgrade_method_needed(39, 39):
+            return
+        for ks_name in ('keystore', *['x{}/'.format(i) for i in range(1, 16)]):
+            ks = self.data.get(ks_name, None)
+            if ks is None: continue
+            seed = ks.get('seed')
+            if not seed: continue
+            seed_type = None
+            xpub = ks.get('xpub') or None
+            if xpub:
+                assert isinstance(xpub, str)
+                if xpub[0:4] in ('xpub', 'tpub'):
+                    seed_type = 'standard'
+                elif xpub[0:4] in ('zpub', 'Zpub', 'vpub', 'Vpub'):
+                    seed_type = 'segwit'
+            elif ks.get('type') == 'old':
+                seed_type = 'old'
+            if seed_type is not None:
+                ks['seed_type'] = seed_type
+        self.data['seed_version'] = 40
+
     def _convert_imported(self):
         if not self._is_upgrade_method_needed(0, 13):
             return
@@ -911,6 +1027,8 @@ class WalletDB(JsonDB):
         assert isinstance(tx_hash, str)
         assert isinstance(tx, Transaction), tx
         # note that tx might be a PartialTransaction
+        # serialize and de-serialize tx now. this might e.g. convert a complete PartialTx to a Tx
+        tx = tx_from_any(str(tx))
         if not tx_hash:
             raise Exception("trying to add tx to db without txid")
         if tx_hash != tx.txid():
@@ -1188,8 +1306,10 @@ class WalletDB(JsonDB):
             v = dict((k, FeeUpdate(**x)) for k, x in v.items())
         elif key == 'submarine_swaps':
             v = dict((k, SwapData(**x)) for k, x in v.items())
-        elif key == 'channel_backups':
-            v = dict((k, ChannelBackupStorage(**x)) for k, x in v.items())
+        elif key == 'imported_channel_backups':
+            v = dict((k, ImportedChannelBackupStorage(**x)) for k, x in v.items())
+        elif key == 'onchain_channel_backups':
+            v = dict((k, OnchainChannelBackupStorage(**x)) for k, x in v.items())
         elif key == 'tx_fees':
             v = dict((k, TxFeesValue(*x)) for k, x in v.items())
         elif key == 'prevouts_by_scripthash':
@@ -1223,13 +1343,15 @@ class WalletDB(JsonDB):
         with self.lock:
             self._write(storage)
 
+    @profiler
     def _write(self, storage: 'WalletStorage'):
         if threading.currentThread().isDaemon():
             self.logger.warning('daemon thread cannot write db')
             return
         if not self.modified():
             return
-        storage.write(self.dump())
+        json_str = self.dump(human_readable=not storage.is_encrypted())
+        storage.write(json_str)
         self.set_modified(False)
 
     def is_ready_to_be_used_by_wallet(self):
